/*
DL2 Monitor: ./pa/papap.c
Copyright (C) 2002, Rolf Jger, Ulm (Germany)

See the file README for information on usage and redistribution
of this source code file, and for a DISCLAIMER OF ALL WARRANTIES.
*/

/*----------------------------------------------------------------------------
 * Datalink Monitor (papap): Protocol Analysis (RFC1334)
 * Password Authentication Protocol
 *
 * Implementation Notes
 * P_* indicate frame positions of certain fields;
 * C_* indicate known values of the code field;
 * default contains: code, request-id
 * 
 * Author: Rolf Jger,Ulm
 *----------------------------------------------------------------------------
 * YYYYMMDD  History
 * 20020427  length instad of packet-length.
 * 20020405  default selection extended by request-id.
 * 20020330  created from scratch
 * $Id: papap.c,v 1.5 2002/04/29 12:59:15 dv Exp $
 *----------------------------------------------------------------------------
*/
# include <stdio.h>
# include <string.h>
# include "lib/dl2pa.h"
# include "pa/pap.h"

// PAP Fram Positions:
# define P_CODE 0x00
# define P_REQUESTID 0x01
# define P_LENGTH 0x02
# define P_DATA 0x04
// {<length> <data>}, with 1 octet per <length> 

// known codes: 1..3
# define C_Arequest  0x01
# define C_Aack      0x02
# define C_Anak      0x03

// default indications:
static const char* codestr[] = {
  "",
  "AuthRequest",
  "AuthAck",
  "AuthNak"
};
# define MAXCODE C_Anak

//-exports--------------------------------------------------------------------
int dl2_papap(Dl2_Frame *frame,int verbose) {
  Octet *buffer = frame->buffer;
  int length = frame->length, i = 0;
  Octet code;
  Doctet pl;  

  // check for required packet length:
  if (length < P_DATA) return -1;

  //code:
  code = buffer[i];
  if (verbose & PAP_Mcode) {
    printf("%s=%02x",PAP_Fcode,code);
  } 
  else if (verbose & PAP_Mdefault) {
    const char *cs = (code <= MAXCODE ? codestr[code] : PA_FC_UNKNOWN); 
    printf(" %s",codestr[code]); 
  } i = P_REQUESTID;

  // request-id:
  if (verbose & PAP_Mrequest_id) {
    printf(" %s=%02x",PAP_Frequest_id,buffer[i]);
  }
  else if (verbose & PAP_Mdefault) {
    printf("(%02x)",buffer[i]);
  } i = P_LENGTH;

  // packet-length:
  if (verbose & PAP_Mlength) {
    printf(" %s=",PAP_Flength);
    for (; i < P_DATA; i++) printf("%02x",buffer[i]);
  } else i = P_DATA;

  // data:
  if ((pl = str2doct(buffer+P_LENGTH)) > length) pl = length;
  if (verbose & PAP_Mdata) {
    printf(" %s=",PAP_Fdata);
    for (; i < pl; i++) printf("%02x",buffer[i]);
  } else i = pl;

  if (code==C_Arequest) {

    // peer:
    i = P_DATA+1; pl = i + buffer[P_DATA];
    if (verbose & PAP_Mpeer) {
      printf(" %s=",PAP_Fpeer);
      for (; i < pl; i++) printf("%02x",buffer[i]);
    } else i = pl;

    // password:
    pl += buffer[i]+1;
    if (verbose & PAP_Mpassword) {
      printf(" %s=",PAP_Fpassword);
      for (i += 1; i < pl; i++) printf("%02x",buffer[i]);
    } else i = pl;
  }
  else if (code==C_Aack || code==C_Anak) {

    // message:
    i = P_DATA+1; pl = i + buffer[P_DATA];
    if (verbose & PAP_Mmessage) {
      printf(" %s=",PAP_Fmessage);
      for (; i < pl; i++) printf("%02x",buffer[i]);
    } i = pl;
  }
  return i;
} //dl2_papap


//-locals---------------------------------------------------------------------

/***EOM***/
